package com.sample;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.RuleBaseConfiguration;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.KnowledgeBaseConfiguration;

/**
 * This is a sample class to launch a rule.
 */
public class DroolsTest {

	public static final void main(String[] args) {
		boolean reversed = args.length > 0;
		try {
			// load up the knowledge base
			KnowledgeBase kbase = readKnowledgeBase();
			StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newFileLogger(ksession, "test");
			// go !
			if(reversed) {
				System.out.println("Reversed");
				ksession.insert(new Move(2, 3));
			}
			ksession.insert(new Move(1, 2));
			if(!reversed) {
				System.out.println("Normal");
				ksession.insert(new Move(2, 3));
			}
			
			ksession.fireAllRules();
			logger.close();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	@SuppressWarnings("restriction")
	private static KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("Sample.drl"), ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error: errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBaseConfiguration conf = new RuleBaseConfiguration();
		conf.setProperty("drools.conflictResolver", "com.sample.CustomConflictResolver");
		
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(conf);
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}

	public static class Move {
		
		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + first;
			result = prime * result + second;
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			Move other = (Move) obj;
			if (first != other.first)
				return false;
			if (second != other.second)
				return false;
			return true;
		}

		public final int first;
		public final int second;

		public Move(int first, int second) {
			this.first = first;
			this.second = second;
		}
		
		public int getFirst() {
			return this.first;
		}
		
		public int getSecond() {
			return this.second;
		}
		
		public String toString() {
			return "Move(" + first + "," + second + ")";
		}
	}

	public static class Win {
		
		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + first;
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			Win other = (Win) obj;
			if (first != other.first)
				return false;
			return true;
		}

		public final int first;

		public Win(int first) {
			this.first = first;
		}
		
		public int getFirst() {
			return this.first;
		}
		
		public String toString() {
			return "Win(" + first + ")";
		}
	}

}